package sun.awt.windows;

import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.awt.Component;
import sun.awt.im.InputMethodManager;
import java.awt.MenuBar;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Frame;
import sun.awt.AWTAccessor;
import java.awt.peer.FramePeer;

class WFramePeer extends WWindowPeer implements FramePeer
{
    private static final boolean keepOnMinimize;
    
    private static native void initIDs();
    
    @Override
    public native void setState(int p0);
    
    @Override
    public native int getState();
    
    public void setExtendedState(int n) {
        AWTAccessor.getFrameAccessor().setExtendedState((Frame)this.target, n);
    }
    
    public int getExtendedState() {
        return AWTAccessor.getFrameAccessor().getExtendedState((Frame)this.target);
    }
    
    private native void setMaximizedBounds(int p0, int p1, int p2, int p3);
    
    private native void clearMaximizedBounds();
    
    @Override
    public void setMaximizedBounds(Rectangle rectangle) {
        if (rectangle == null) {
            this.clearMaximizedBounds();
        }
        else {
            Rectangle rectangle2 = (Rectangle)rectangle.clone();
            this.adjustMaximizedBounds(rectangle2);
            this.setMaximizedBounds(rectangle2.x, rectangle2.y, rectangle2.width, rectangle2.height);
        }
    }
    
    private void adjustMaximizedBounds(Rectangle rectangle) {
        GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration();
        GraphicsConfiguration defaultConfiguration = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        if (graphicsConfiguration != null && graphicsConfiguration != defaultConfiguration) {
            Rectangle bounds = graphicsConfiguration.getBounds();
            Rectangle bounds2 = defaultConfiguration.getBounds();
            if (bounds.width - bounds2.width > 0 || bounds.height - bounds2.height > 0) {
                rectangle.width -= bounds.width - bounds2.width;
                rectangle.height -= bounds.height - bounds2.height;
            }
        }
    }
    
    @Override
    public boolean updateGraphicsData(GraphicsConfiguration graphicsConfiguration) {
        int updateGraphicsData = super.updateGraphicsData(graphicsConfiguration);
        Rectangle maximizedBounds = AWTAccessor.getFrameAccessor().getMaximizedBounds((Frame)this.target);
        if (maximizedBounds != null) {
            this.setMaximizedBounds(maximizedBounds);
        }
        return updateGraphicsData;
    }
    
    @Override
    boolean isTargetUndecorated() {
        return ((Frame)this.target).isUndecorated();
    }
    
    @Override
    public void reshape(int n, int n2, int n3, int n4) {
        if (((Frame)this.target).isUndecorated()) {
            super.reshape(n, n2, n3, n4);
        }
        else {
            this.reshapeFrame(n, n2, n3, n4);
        }
    }
    
    @Override
    public Dimension getMinimumSize() {
        Dimension dimension = new Dimension();
        if (!((Frame)this.target).isUndecorated()) {
            dimension.setSize(WWindowPeer.getSysMinWidth(), WWindowPeer.getSysMinHeight());
        }
        if (((Frame)this.target).getMenuBar() != null) {
            dimension.height += getSysMenuHeight();
            return dimension;
        }
        return dimension;
    }
    
    @Override
    public void setMenuBar(MenuBar menuBar) {
        WMenuBarPeer menuBar2 = (WMenuBarPeer)WToolkit.targetToPeer(menuBar);
        if (menuBar2 != null) {
            if (menuBar2.framePeer != this) {
                menuBar.removeNotify();
                menuBar.addNotify();
                menuBar2 = (WMenuBarPeer)WToolkit.targetToPeer(menuBar);
                if (menuBar2 != null && menuBar2.framePeer != this) {
                    throw new IllegalStateException("Wrong parent peer");
                }
            }
            if (menuBar2 != null) {
                this.addChildPeer(menuBar2);
            }
        }
        this.setMenuBar0(menuBar2);
        this.updateInsets(this.insets_);
    }
    
    private native void setMenuBar0(WMenuBarPeer p0);
    
    WFramePeer(Frame frame) {
        super(frame);
        String triggerMenuString = InputMethodManager.getInstance().getTriggerMenuString();
        if (triggerMenuString != null) {
            this.pSetIMMOption(triggerMenuString);
        }
    }
    
    native void createAwtFrame(WComponentPeer p0);
    
    @Override
    void create(WComponentPeer wComponentPeer) {
        this.preCreate(wComponentPeer);
        this.createAwtFrame(wComponentPeer);
    }
    
    @Override
    void initialize() {
        super.initialize();
        Frame frame = (Frame)this.target;
        if (frame.getTitle() != null) {
            this.setTitle(frame.getTitle());
        }
        this.setResizable(frame.isResizable());
        this.setState(frame.getExtendedState());
    }
    
    private static native int getSysMenuHeight();
    
    native void pSetIMMOption(String p0);
    
    void notifyIMMOptionChange() {
        InputMethodManager.getInstance().notifyChangeRequest((Component)this.target);
    }
    
    @Override
    public void setBoundsPrivate(int n, int n2, int n3, int n4) {
        this.setBounds(n, n2, n3, n4, 3);
    }
    
    @Override
    public Rectangle getBoundsPrivate() {
        return this.getBounds();
    }
    
    @Override
    public void emulateActivation(boolean b) {
        this.synthesizeWmActivate(b);
    }
    
    private native void synthesizeWmActivate(boolean p0);
    
    static {
        initIDs();
        keepOnMinimize = "true".equals(AccessController.doPrivileged(new GetPropertyAction("sun.awt.keepWorkingSetOnMinimize")));
    }
}
